"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.APIConfig = void 0;
const code_morph_ts_1 = require("@fresha/code-morph-ts");
const openapi_codegen_utils_1 = require("@fresha/openapi-codegen-utils");
const ts_morph_1 = require("ts-morph");
const operations_1 = require("./operations");
// type APIConfig = {
//   rootUrl: APIRootURL;
//   entries: APIEntriesConfig;
// };
// type APIClientConfig = Record<APIRootURL, APIConfig>;
/**
 * Generates API initialization function, as well as configured API variable.
 */
class APIConfig {
    constructor(context, sourceFile) {
        this.context = context;
        this.sourceFile = sourceFile;
        this.apiConfig = {};
    }
    collectData() {
        for (const operation of (0, openapi_codegen_utils_1.getOperations)(this.context.openapi)) {
            this.context.logger.info(`Processing path item: "${operation.parent.pathUrl}"`);
            const entryKey = (0, openapi_codegen_utils_1.getOperationEntryKeyOrThrow)(operation);
            let apiEntryConfig = this.apiConfig[entryKey];
            if (!apiEntryConfig) {
                apiEntryConfig = {
                    url: (0, openapi_codegen_utils_1.pathUrlToUrlExp)(operation.parent.pathUrl),
                    operations: [],
                };
                this.apiConfig[entryKey] = apiEntryConfig;
            }
            const operationId = (0, openapi_codegen_utils_1.getOperationIdOrThrow)(operation);
            const { template, name } = (0, operations_1.findOperationTemplate)(operation.httpMethod, operation.parent.pathUrl, entryKey, operationId);
            const queryParams = operation.parameters?.filter(p => p.in === 'query').map(p => p.name);
            const templateActionName = template.actionName(entryKey);
            let operConfig = name;
            if (templateActionName !== operationId) {
                if (queryParams?.length) {
                    operConfig = [name, { queryParams }];
                }
                else {
                    operConfig = [name, { name: operationId }];
                }
            }
            else if (queryParams?.length) {
                operConfig = [name, { queryParams }];
            }
            apiEntryConfig.operations.push(operConfig);
        }
    }
    generateCode() {
        this.context.logger.info(`Found configuration for ${this.context.apiRootUrl} ${JSON.stringify(this.apiConfig, null, 2)}`);
        (0, code_morph_ts_1.addImportDeclarations)(this.sourceFile, {
            '@fresha/connector-utils/build/types/api': ['APIEnvironmentOptions'],
            '@fresha/redux-store': ['.:store'],
            '@fresha/connector-utils/build/apiConfig': ['configureApi'],
            '@fresha/connector-utils/build/boundActions': ['boundActions'],
            '@fresha/connector-utils/build/env': ['env'],
        });
        this.generateMakeApiConfig();
        (0, code_morph_ts_1.addConstant)(this.sourceFile, 'apiConfig', 'makeApiConfig(env)');
        (0, code_morph_ts_1.addConstant)(this.sourceFile, 'configuredApi', `configureApi([apiConfig], '${this.context.apiName}')`, true);
    }
    generateMakeApiConfig() {
        const initFunctionDecl = (0, code_morph_ts_1.addFunction)(this.sourceFile, 'makeApiConfig', {
            [`{ ${this.context.apiRootUrl} }`]: `Pick<APIEnvironmentOptions, '${this.context.apiRootUrl}'>`,
        }, undefined, true);
        initFunctionDecl.addVariableStatement({
            declarationKind: ts_morph_1.VariableDeclarationKind.Const,
            declarations: [
                {
                    name: 'config',
                    initializer: '[{}, {}] as const',
                },
            ],
        });
        initFunctionDecl.addStatements('return config;');
        const configElementsObj = initFunctionDecl
            .getVariableDeclarationOrThrow('config')
            .getInitializerIfKindOrThrow(ts_morph_1.SyntaxKind.AsExpression)
            .getExpressionIfKindOrThrow(ts_morph_1.SyntaxKind.ArrayLiteralExpression)
            .getElements();
        this.generateEntryConfigs(configElementsObj[0]);
        this.generateCommonConfig(configElementsObj[1]);
    }
    generateEntryConfigs(configElementObj) {
        const entriesConfigObj = configElementObj.asKindOrThrow(ts_morph_1.SyntaxKind.ObjectLiteralExpression);
        for (const [entryKey, entryConfig] of Object.entries(this.apiConfig)) {
            const entryObj = (0, code_morph_ts_1.addObjectLiteralObjectProperty)(entriesConfigObj, entryKey);
            (0, code_morph_ts_1.addObjectLiteralProperty)(entryObj, 'url', `'${entryConfig.url}'`);
            const operationsObj = (0, code_morph_ts_1.addObjectLiteralArrayProperty)(entryObj, 'operations');
            for (const operConfig of entryConfig.operations) {
                if (typeof operConfig === 'string') {
                    operationsObj.addElement(`'${operConfig}'`);
                }
                else {
                    const [templateName, overrides] = operConfig;
                    const overridesObj = operationsObj
                        .addElement(`['${templateName}', {}]`)
                        .asKindOrThrow(ts_morph_1.SyntaxKind.ArrayLiteralExpression)
                        .getElements()[1]
                        .asKindOrThrow(ts_morph_1.SyntaxKind.ObjectLiteralExpression);
                    if (overrides.name) {
                        (0, code_morph_ts_1.addObjectLiteralProperty)(overridesObj, 'actionName', `'${overrides.name}'`);
                    }
                    if (overrides.queryParams?.length) {
                        const queryParamsObj = (0, code_morph_ts_1.addObjectLiteralArrayProperty)(overridesObj, 'queryParams');
                        for (const paramName of overrides.queryParams) {
                            queryParamsObj.addElement(`'${paramName}'`);
                        }
                    }
                }
            }
        }
    }
    generateCommonConfig(configElementObj) {
        (0, code_morph_ts_1.addObjectLiteralProperties)(configElementObj.asKindOrThrow(ts_morph_1.SyntaxKind.ObjectLiteralExpression), {
            rootUrl: this.context.apiRootUrl,
            adapter: `'${this.context.useJsonApi ? 'jsonapi' : 'raw'}'`,
        });
    }
}
exports.APIConfig = APIConfig;
//# sourceMappingURL=APIConfig.js.map