"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ActionSignature = void 0;
const assert_1 = __importDefault(require("assert"));
const api_tools_core_1 = require("@fresha/api-tools-core");
const code_morph_ts_1 = require("@fresha/code-morph-ts");
const openapi_codegen_utils_1 = require("@fresha/openapi-codegen-utils");
const ts_morph_1 = require("ts-morph");
// returns true if action needs args argument (everything except resource ID)
const actionNeedsArgs = (operation) => {
    for (const param of operation.parameters) {
        if (param.name !== 'id' || param.in !== 'path') {
            return true;
        }
    }
    if (operation.requestBody?.content) {
        for (const requestMimeType of operation.requestBody.content.values()) {
            if (requestMimeType.schema != null) {
                return true;
            }
        }
    }
    return false;
};
class ActionSignature {
    constructor(context, name, operation, template) {
        this.context = context;
        this.operation = operation;
        this.template = template;
        this.name = name;
        this.requestTypeName = (0, api_tools_core_1.titleCase)(`${this.name}Request`);
        this.responseTypeName = (0, api_tools_core_1.titleCase)(`${this.name}Response`);
    }
    generateCode(type) {
        this.context.logger.info(`Generating type for action ${this.name}`);
        const sourceFile = type.getSourceFile();
        const actionType = (0, code_morph_ts_1.addTypeLiteralProperty)(type, this.name, '{}').asKindOrThrow(ts_morph_1.SyntaxKind.TypeLiteral);
        const actionFuncTypeObj = (0, code_morph_ts_1.addTypeLiteralCall)(actionType, {}, 'Promise<Response>');
        if (this.operation.getParameter('id', 'path')) {
            if (!sourceFile.getTypeAlias('ResourceId')) {
                (0, code_morph_ts_1.addTypeAlias)(sourceFile, 'ResourceId', 'string | number');
            }
            actionFuncTypeObj.addParameter({
                name: 'id',
                type: 'ResourceId',
            });
        }
        if (actionNeedsArgs(this.operation)) {
            const argsParamType = actionFuncTypeObj
                .addParameter({
                name: 'args',
                type: '{}',
            })
                .getNodeProperty('type')
                ?.asKindOrThrow(ts_morph_1.SyntaxKind.TypeLiteral);
            (0, assert_1.default)(argsParamType);
            for (const param of this.operation.parameters) {
                if (param.name !== 'id' || param.in !== 'path') {
                    const p = (0, code_morph_ts_1.addTypeLiteralProperty)(argsParamType, param.name, (writer) => {
                        switch (param.schema?.type) {
                            case undefined:
                                writer.write('unknown');
                                break;
                            case null:
                            case 'string':
                                writer.write(param.schema?.nullable ? 'string | null' : 'string');
                                break;
                            case 'number':
                            case 'integer':
                                writer.write(param.schema?.nullable ? 'number | null' : 'number');
                                break;
                            case 'boolean':
                                writer.write(param.schema?.nullable ? 'boolean | null' : 'boolean');
                                break;
                            default:
                                assert_1.default.fail(`Unexpected schema type ${String(param.schema?.type)} in parameter ${param.name}:${param.in}`);
                        }
                    });
                    p.getParentIfKindOrThrow(ts_morph_1.SyntaxKind.PropertySignature).setHasQuestionToken(!param.required);
                }
            }
            // add request body
            const requestBodySchema = (0, openapi_codegen_utils_1.getOperationRequestBodySchema)(this.operation, this.context.useJsonApi);
            if (requestBodySchema) {
                this.addRequestBodyParams(argsParamType, requestBodySchema);
            }
        }
        (0, code_morph_ts_1.addTypeLiteralProperty)(actionType, 'apiName', `'${this.name}'`);
        const actionCache = this.operation.getExtension('cache');
        (0, code_morph_ts_1.addTypeLiteralProperty)(actionType, 'cache', actionCache != null ? 'Map<string, unknown>' : 'null');
    }
    addRequestBodyParams(argsParamType, schema) {
        (0, assert_1.default)(schema.type === 'object');
        for (const prop of schema.getPropertiesDeep()) {
            switch (prop.schema.type) {
                case null:
                    if (prop.schema.allOf?.length) {
                        const allowedType = new Set(prop.schema.allOf.map(subschema => subschema.type));
                        (0, assert_1.default)(allowedType.size === 1 && allowedType.has('object'), `For composite schemas, expected to have identical schema type. ${prop.name}`);
                        const propType = (0, code_morph_ts_1.addTypeLiteralProperty)(argsParamType, prop.name, '{}');
                        for (const subschema of prop.schema.allOf) {
                            this.addRequestBodyParams(propType.asKindOrThrow(ts_morph_1.SyntaxKind.TypeLiteral), subschema);
                        }
                    }
                    else {
                        (0, code_morph_ts_1.addImportDeclaration)(argsParamType.getSourceFile(), '@fresha/noname-core', 't:JSONValue');
                        (0, code_morph_ts_1.addTypeLiteralProperty)(argsParamType, prop.name, 'JSONValue');
                    }
                    break;
                case 'boolean':
                    (0, code_morph_ts_1.addTypeLiteralProperty)(argsParamType, prop.name, prop.schema.nullable ? 'boolean | null' : 'boolean');
                    break;
                case 'number':
                case 'integer':
                    (0, code_morph_ts_1.addTypeLiteralProperty)(argsParamType, prop.name, prop.schema.nullable ? 'number | null' : 'number');
                    break;
                case 'string': {
                    let propType = 'string';
                    if (prop.schema.enum) {
                        if (prop.schema.enum.length === 1) {
                            propType = `'${String(prop.schema.enum[0])}'`;
                        }
                        else if (prop.schema.enum.length > 1) {
                            propType = prop.schema.enum?.map(elem => `'${String(elem)}'`).join(' | ');
                        }
                        if (prop.schema.nullable) {
                            propType = `${propType} | null`;
                        }
                    }
                    (0, code_morph_ts_1.addTypeLiteralProperty)(argsParamType, prop.name, propType);
                    break;
                }
                case 'object': {
                    if (prop.schema.properties.size) {
                        const propType = (0, code_morph_ts_1.addTypeLiteralProperty)(argsParamType, prop.name, '{}');
                        this.addRequestBodyParams(propType.asKindOrThrow(ts_morph_1.SyntaxKind.TypeLiteral), prop.schema);
                    }
                    else {
                        (0, code_morph_ts_1.addImportDeclaration)(argsParamType.getSourceFile(), '@fresha/noname-core', 't:JSONObject');
                        (0, code_morph_ts_1.addTypeLiteralProperty)(argsParamType, prop.name, 'JSONObject');
                    }
                    break;
                }
                case 'array':
                    (0, code_morph_ts_1.addImportDeclaration)(argsParamType.getSourceFile(), '@fresha/noname-core', 't:JSONArray');
                    (0, code_morph_ts_1.addTypeLiteralProperty)(argsParamType, prop.name, 'JSONArray');
                    break;
                default:
                    this.context.logger.warn(`Cannot handle schema type ${String(prop.schema.type)}`);
            }
            argsParamType
                .getPropertyOrThrow(prop.name)
                .setHasQuestionToken(!schema.required.has(prop.name));
        }
    }
}
exports.ActionSignature = ActionSignature;
//# sourceMappingURL=ActionSignature.js.map